@@ -3,7 +3,24 @@
 contTablesClass <- R6::R6Class(
     "contTablesClass",
     inherit=contTablesBase,
+    #### Active bindings ----
+    active = list(
+        countsName = function() {
+            if (is.null(private$.countsName)) {
+                analysisCounts <- self$options$counts
+                if ( ! is.null(analysisCounts))
+                    private$.countsName <- analysisCounts
+                else if ( ! is.null(attr(self$data, "jmv-weights"))) {
+                    private$.countsName <- ".COUNTS"
+                }
+            }
+
+            return(private$.countsName)
+        }
+    ),
     private=list(
+        #### Member variables ----
+        .countsName = NULL,
         #### Init + run functions ----
         .init=function() {
 
@@ -221,7 +238,7 @@ contTablesClass <- R6::R6Class(
 
             rowVarName <- self$options$rows
             colVarName <- self$options$cols
-            countsName <- self$options$counts
+            countsName <- self$countsName
 
             if (is.null(rowVarName) || is.null(colVarName))
                 return()
@@ -325,6 +342,14 @@ contTablesClass <- R6::R6Class(
                     dp <- NULL
                     lor <- NULL
                     fish <- try(stats::fisher.test(mat, conf.level=ciWidth, alternative=Ha), silent=TRUE)
+
+                    if (base::inherits(fish, 'try-error')) {
+                        # Monte Carlo simulation for p-value
+                        fish <- try(stats::fisher.test(mat, alternative=Ha, simulate.p.value=TRUE), silent=TRUE)
+                        MCpsimul <- TRUE
+                    } else
+                        MCpsimul <- FALSE
+
                     if (all(dim(mat) == 2) && all(rowSums(mat) > 0) && all(colSums(mat) > 0)) {
                         dp <- private$.diffProp(mat, Ha)
                         lor <- vcd::loddsratio(mat)
@@ -425,11 +450,10 @@ contTablesClass <- R6::R6Class(
                         `value[N]`=n)
                 } else {
 
-                    if (inherits(fish, 'htest')) {
+                    if (base::inherits(fish, 'try-error'))
+                        fishP <- NaN
+                    else
                         fishP <- fish$p.value
-                    } else {
-                        fishP <- ''
-                    }
 
                     if (is.null(zP)) {
                         zPstat <- NaN
@@ -475,6 +499,9 @@ contTablesClass <- R6::R6Class(
                 if (inherits(fish, 'htest') && all(dim(mat) == 2) && hypothesis != "different")
                     chiSq$addFootnote(rowNo=othRowNo, 'p[fisher]', hypothesisTested)
 
+                if (MCpsimul)
+                    chiSq$addFootnote(rowNo=othRowNo, 'p[fisher]', .('Monte Carlo simulation'))
+
                 values <- list(
                     `v[cont]`=asso$contingency,
                     `v[phi]`=ifelse(is.na(asso$phi), NaN, asso$phi),
@@ -528,11 +555,11 @@ contTablesClass <- R6::R6Class(
                         `v[rr]`=rr$rr,
                         `cil[rr]`=rr$lower,
                         `ciu[rr]`=rr$upper))
-                    
+
                     footnote <- `if`(self$options$compare == 'rows', .('Rows compared'), .('Columns compared'))
                     odds$addFootnote(rowNo=othRowNo, 'v[dp]', footnote)
                     odds$addFootnote(rowNo=othRowNo, 'v[rr]', footnote)
-                    
+
                     if (any(mat == 0)){
                         odds$addFootnote(rowNo=othRowNo, 'v[lo]', .('Haldane-Anscombe correction applied'))
                         odds$addFootnote(rowNo=othRowNo, 'v[o]', .('Haldane-Anscombe correction applied'))
@@ -579,12 +606,12 @@ contTablesClass <- R6::R6Class(
             if (! is.null(colVarName))
                 colVarName <- jmvcore::toB64(colVarName)
 
-            countsName <- self$options$counts
+            countsName <- self$countsName
             if (! is.null(countsName))
                 countsName <- jmvcore::toB64(countsName)
 
             layerNames <- self$options$layers
-            if (! is.null(layerNames))
+            if (length(layerNames) > 0)
                 layerNames <- jmvcore::toB64(layerNames)
             if (length(layerNames) > 2)
                 layerNames <- layerNames[1:2] # max 2
@@ -595,12 +622,7 @@ contTablesClass <- R6::R6Class(
             data <- private$.cleanData(B64 = TRUE)
             data <- na.omit(data)
 
-            if (! is.null(countsName)){
-                untable <- function (df, counts) df[rep(1:nrow(df), counts), ]
-                data <- untable(data[, c(rowVarName, colVarName, layerNames)], counts=data[, countsName])
-            }
-
-            formula <- jmvcore::composeFormula(NULL, c(rowVarName, colVarName, layerNames))
+            formula <- jmvcore::composeFormula(countsName, c(rowVarName, colVarName, layerNames))
             counts <- xtabs(formula, data)
             d <- dim(counts)
 
@@ -665,7 +687,7 @@ contTablesClass <- R6::R6Class(
 
             p <- p + labs(x=jmvcore::fromB64(xVarName), fill=jmvcore::fromB64(zVarName))
 
-            if (! is.null(layerNames)) {
+            if (length(layerNames) > 0) {
                 if (length(layerNames) == 1)
                     layers <- as.formula(jmvcore::composeFormula(NULL, layerNames))
                 else
@@ -680,7 +702,6 @@ contTablesClass <- R6::R6Class(
 
         #### Helper functions ----
         .cleanData = function(B64 = FALSE) {
-
             data <- self$data
 
             rowVarName <- self$options$rows
@@ -703,6 +724,9 @@ contTablesClass <- R6::R6Class(
             if ( ! is.null(countsName)) {
                 countsNameNew <- ifelse(B64, jmvcore::toB64(countsName), countsName)
                 data[[countsNameNew]] <- jmvcore::toNumeric(data[[countsName]])
+            } else if ( ! is.null(attr(data, "jmv-weights"))) {
+                countsNameNew <- ifelse(B64, jmvcore::toB64(".COUNTS"), ".COUNTS")
+                data[[countsNameNew]] = jmvcore::toNumeric(attr(data, "jmv-weights"))
             }
 
             return(data)
@@ -714,7 +738,7 @@ contTablesClass <- R6::R6Class(
             rowVarName <- self$options$rows
             colVarName <- self$options$cols
             layerNames <- self$options$layers
-            countsName <- self$options$counts
+            countsName <- self$countsName
 
             if (length(layerNames) == 0) {
 
